@@ -15,7 +15,7 @@
  * the sats, one presumes) is 0sec; for the u-blox 6 it's 15sec.
  *
  * This file is Copyright (c) 2010 by the GPSD project
- * BSD terms apply: see the file COPYING in the distribution root for details.
+ * SPDX-License-Identifier: BSD-2-clause
  *
  */
 
@@ -62,6 +62,8 @@ static gps_mask_t ubx_parse(struct gps_device_t *session, unsigned char *buf,
 static gps_mask_t ubx_msg_nav_dop(struct gps_device_t *session,
 				  unsigned char *buf, size_t data_len);
 static void ubx_msg_inf(struct gps_device_t *session, unsigned char *buf, size_t data_len);
+static gps_mask_t ubx_msg_nav_posecef(struct gps_device_t *session,
+				      unsigned char *buf, size_t data_len);
 static gps_mask_t ubx_msg_nav_pvt(struct gps_device_t *session,
 				  unsigned char *buf, size_t data_len);
 static void ubx_msg_sbas(struct gps_device_t *session, unsigned char *buf);
@@ -71,6 +73,8 @@ static gps_mask_t ubx_msg_nav_svinfo(struct gps_device_t *session,
 				     unsigned char *buf, size_t data_len);
 static gps_mask_t ubx_msg_nav_timegps(struct gps_device_t *session,
 				      unsigned char *buf, size_t data_len);
+static gps_mask_t ubx_msg_nav_velecef(struct gps_device_t *session,
+				      unsigned char *buf, size_t data_len);
 static void ubx_msg_mon_ver(struct gps_device_t *session,
 				      unsigned char *buf, size_t data_len);
 #ifdef RECONFIGURE_ENABLE
@@ -123,6 +127,37 @@ ubx_msg_mon_ver(struct gps_device_t *session, unsigned char *buf,
              (int)sizeof(obuf), obuf);
 }
 
+/*
+ * Navigation Position ECEF message
+ */
+static gps_mask_t
+ubx_msg_nav_posecef(struct gps_device_t *session, unsigned char *buf,
+		size_t data_len)
+{
+    gps_mask_t mask = ECEF_SET;
+    double fTOW;
+
+    if (data_len < 20) {
+	gpsd_log(&session->context->errout, LOG_WARN,
+		 "Invalid NAV POSECEF message, payload len %zd", data_len);
+	return 0;
+    }
+
+    fTOW = getleu32(buf, 0) / 1000.0;
+    session->newdata.ecef.x = getles32(buf, 4) / 100.0;
+    session->newdata.ecef.y = getles32(buf, 8) / 100.0;
+    session->newdata.ecef.z = getles32(buf, 12) / 100.0;
+    session->newdata.ecef.pAcc = getleu32(buf, 16) / 100.0;
+    gpsd_log(&session->context->errout, LOG_DATA,
+	"UBX_NAV_POSECEF: fTOW=%.3f ECEF x=%.2f y=%.2f z=%.2f pAcc=%.2f\n",
+	fTOW,
+	session->newdata.ecef.x,
+	session->newdata.ecef.y,
+	session->newdata.ecef.z,
+	session->newdata.ecef.pAcc);
+    return mask;
+}
+
 /**
  * Navigation Position Velocity Time  solution message
  */
@@ -130,31 +165,139 @@ static gps_mask_t
 ubx_msg_nav_pvt(struct gps_device_t *session, unsigned char *buf,
 		size_t data_len)
 {
-    unsigned int flags;
+    uint8_t valid;
+    uint8_t flags;
+    uint8_t navmode;
+    struct tm unpacked_date;
+    double hacc, vacc, sacc;
+    int *status = &session->gpsdata.status;
+    int *mode = &session->newdata.mode;
     gps_mask_t mask = 0;
 
     if (data_len != 92)
 	return 0;
 
+    valid = (unsigned int)getub(buf, 11);
+    navmode = (unsigned char)getub(buf, 20);
     flags = (unsigned int)getub(buf, 21);
 
-    /* TODO: finish decoding UBX_MON_PVT
-     * no need until depreacaed UBX_MON_SOL is dead
-     */
+    switch (navmode)
+    {
+    case UBX_MODE_TMONLY:
+    {
+	if (*mode != MODE_NO_FIX) {
+	    *mode = MODE_NO_FIX;
+	    mask |= MODE_SET;
+	}
+	if (*status != STATUS_NO_FIX) {
+	    *status = STATUS_NO_FIX;
+	    mask |= STATUS_SET;
+	}
+	break;
+    }
+    case UBX_MODE_3D:
+    case UBX_MODE_GPSDR:
+    {
+	if (*mode != MODE_3D) {
+	    *mode = MODE_3D;
+	    mask |= MODE_SET;
+	}
+	if ((flags & UBX_NAV_PVT_FLAG_DGPS) == UBX_NAV_PVT_FLAG_DGPS) {
+	    if (*status != STATUS_DGPS_FIX) {
+		*status = STATUS_DGPS_FIX;
+		mask |= STATUS_SET;
+	    }
+	} else {
+	    if (*status != STATUS_FIX) {
+		*status = STATUS_FIX;
+		mask |= STATUS_SET;
+	    }
+	}
+	mask |=	  LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET;
+	break;
+    }
+    case UBX_MODE_2D:
+    case UBX_MODE_DR:		/* consider this too as 2D */
+    {
+	if (*mode != MODE_2D) {
+	    *mode = MODE_2D;
+	    mask |= MODE_SET;
+	};
+	if (*status != STATUS_FIX) {
+	    *status = STATUS_FIX;
+	    mask |= STATUS_SET;
+	}
+	mask |= LATLON_SET | SPEED_SET;
+	break;
+    }
+    default:
+    {
+	if (*mode != MODE_NO_FIX) {
+	    *mode = MODE_NO_FIX;
+	    mask |= MODE_SET;
+	};
+	if (*status != STATUS_NO_FIX) {
+	    *status = STATUS_NO_FIX;
+	    mask |= STATUS_SET;
+	}
+	break;
+    }
+    }
+
+    if ((valid & UBX_NAV_PVT_VALID_DATE_TIME) == UBX_NAV_PVT_VALID_DATE_TIME) {
+        double subseconds;
+	unpacked_date.tm_year = (uint16_t)getleu16(buf, 4) - 1900;
+	unpacked_date.tm_mon = (uint8_t)getub(buf, 6) - 1;
+	unpacked_date.tm_mday = (uint8_t)getub(buf, 7);
+	unpacked_date.tm_hour = (uint8_t)getub(buf, 8);
+	unpacked_date.tm_min = (uint8_t)getub(buf, 9);
+	unpacked_date.tm_sec = (uint8_t)getub(buf, 10);
+	unpacked_date.tm_isdst = 0;
+	unpacked_date.tm_wday = 0;
+	unpacked_date.tm_yday = 0;
+	subseconds = 1e-9 * (int32_t)getles32(buf, 16);
+	session->newdata.time = \
+	    (timestamp_t)mkgmtime(&unpacked_date) + subseconds;
+	mask |= TIME_SET | NTPTIME_IS;
+    }
+
+    session->newdata.longitude = 1e-7 * (int32_t)getles32(buf, 24);
+    session->newdata.latitude = 1e-7 * (int32_t)getles32(buf, 28);
+    session->newdata.altitude = 1e-3 * (int32_t)getles32(buf, 32);
+    session->newdata.speed = 1e-3 * (int32_t)getles32(buf, 60);
+    session->newdata.track = 1e-5 * (int32_t)getles32(buf, 64);
+    hacc = (double)(getles32(buf, 40) / 1000.0);
+    vacc = (double)(getles32(buf, 44) / 1000.0);
+    sacc = (double)(getles32(buf, 48) / 1000.0);
+    // Assuming hacc == epx == epy is the best we can do
+    session->newdata.epx = session->newdata.epy = hacc;
+    session->newdata.epv = vacc;
+    session->newdata.eps = sacc;
+    mask |= HERR_SET | VERR_SET | SPEEDERR_SET;
     gpsd_log(&session->context->errout, LOG_DATA,
-	     "NAV-PVT: flags:%02x\n", flags);
+	 "NAV_PVT: flags=%02x time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f climb=%.2f mode=%d status=%d used=%d\n",
+	 flags,
+	 session->newdata.time,
+	 session->newdata.latitude,
+	 session->newdata.longitude,
+	 session->newdata.altitude,
+	 session->newdata.track,
+	 session->newdata.speed,
+	 session->newdata.climb,
+	 session->newdata.mode,
+	 session->gpsdata.status,
+	 session->gpsdata.satellites_used);
     return mask;
 }
 
 /**
- * Navigation solution message
+ * Navigation solution message: UBX-NAV-SOL
  */
 static gps_mask_t
 ubx_msg_nav_sol(struct gps_device_t *session, unsigned char *buf,
 		size_t data_len)
 {
     unsigned int flags;
-    double epx, epy, epz, evx, evy, evz;
     unsigned char navmode;
     gps_mask_t mask;
 
@@ -174,18 +317,25 @@ ubx_msg_nav_sol(struct gps_device_t *session, unsigned char *buf,
     }
 #undef DATE_VALID
 
-    epx = (double)(getles32(buf, 12) / 100.0);
-    epy = (double)(getles32(buf, 16) / 100.0);
-    epz = (double)(getles32(buf, 20) / 100.0);
-    evx = (double)(getles32(buf, 28) / 100.0);
-    evy = (double)(getles32(buf, 32) / 100.0);
-    evz = (double)(getles32(buf, 36) / 100.0);
+    session->newdata.ecef.x = getles32(buf, 12) / 100.0;
+    session->newdata.ecef.y = getles32(buf, 16) / 100.0;
+    session->newdata.ecef.z = getles32(buf, 20) / 100.0;
+    session->newdata.ecef.pAcc = getleu32(buf, 24) / 100.0;
+    session->newdata.ecef.vx = getles32(buf, 28) / 100.0;
+    session->newdata.ecef.vy = getles32(buf, 32) / 100.0;
+    session->newdata.ecef.vz = getles32(buf, 36) / 100.0;
+    session->newdata.ecef.vAcc = getleu32(buf, 40) / 100.0;
     ecef_to_wgs84fix(&session->newdata, &session->gpsdata.separation,
-		     epx, epy, epz, evx, evy, evz);
-    mask |= LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET;
+	session->newdata.ecef.x, session->newdata.ecef.y,
+	session->newdata.ecef.z, session->newdata.ecef.vx,
+	session->newdata.ecef.vy, session->newdata.ecef.vz);
+
+    mask |= LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET \
+            | ECEF_SET | VECEF_SET;
 
     if (session->driver.ubx.last_herr > 0.0) {
-	session->newdata.epx = session->newdata.epy = session->driver.ubx.last_herr;
+	session->newdata.epx = session->newdata.epy \
+                             = session->driver.ubx.last_herr;
 	mask |= HERR_SET;
 	session->driver.ubx.last_herr = 0.0;
     }
@@ -206,7 +356,8 @@ ubx_msg_nav_sol(struct gps_device_t *session, unsigned char *buf,
     navmode = (unsigned char)getub(buf, 10);
     switch (navmode) {
     case UBX_MODE_TMONLY:
-	session->newdata.mode = MODE_NO_FIX;
+	/* Surveyed-in, better not have moved */
+	session->newdata.mode = MODE_3D;
 	mask |= GOODTIME_IS;
 	break;
     case UBX_MODE_3D:
@@ -228,7 +379,7 @@ ubx_msg_nav_sol(struct gps_device_t *session, unsigned char *buf,
 
     mask |= MODE_SET | STATUS_SET;
     gpsd_log(&session->context->errout, LOG_DATA,
-	     "NAVSOL: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f climb=%.2f mode=%d status=%d used=%d\n",
+	     "UBX_NAV_SOL: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f climb=%.2f mode=%d status=%d used=%d\n",
 	     session->newdata.time,
 	     session->newdata.latitude,
 	     session->newdata.longitude,
@@ -286,30 +437,48 @@ ubx_msg_nav_dop(struct gps_device_t *session, unsigned char *buf,
 }
 
 /**
- * GPS Leap Seconds
+ * GPS Leap Seconds - UBX-NAV-TIMEGPS
  */
 static gps_mask_t
 ubx_msg_nav_timegps(struct gps_device_t *session, unsigned char *buf,
 		    size_t data_len)
 {
-    unsigned int gw, tow, flags;
+    uint8_t valid;         /* Validity Flags */
+    gps_mask_t mask = 0;
 
     if (data_len != 16)
 	return 0;
 
-    tow = (unsigned int)getleu32(buf, 0);
-    gw = (unsigned int)getles16(buf, 8);
-    flags = (unsigned int)getub(buf, 11);
-    if ((flags & 0x7) != 0)
+    valid = getub(buf, 11);
+    // Valid leap seconds ?
+    if ((valid & UBX_TIMEGPS_VALID_LEAP_SECOND) ==
+        UBX_TIMEGPS_VALID_LEAP_SECOND) {
 	session->context->leap_seconds = (int)getub(buf, 10);
-    session->newdata.time = gpsd_gpstime_resolve(session,
-					      (unsigned short int)gw,
-					      (double)tow / 1000.0);
+    }
+    // Valid GPS time of week and week number
+#define VALID_TIME (UBX_TIMEGPS_VALID_TIME | UBX_TIMEGPS_VALID_WEEK)
+    if ((valid & VALID_TIME) == VALID_TIME) {
+#undef VALID_TIME
+        uint16_t week;
+        double iTOW;      /* integer part of TOW in ms */
+        double fTOW;      /* fractional part of TOW in ns */
+        double TOW;       /* complete TOW in seconds */
+        double tAcc;      /* Time Accuracy Estimate in ns */
+
+	iTOW = (double)getleu32(buf, 0);      /* GPS TOW in ms */
+	fTOW = (double)getles32(buf, 4);      /* Fractional part of TOW */
+	week = getles16(buf, 8);
+	tAcc = (double)getleu32(buf, 12);     /* tAcc in ms */
+        TOW = (iTOW * 1e-3) + (fTOW * 1e-9);
+	session->newdata.time = gpsd_gpstime_resolve(session, week, TOW);
+	session->newdata.ept = tAcc * 1e-9;
+	mask |= (TIME_SET | NTPTIME_IS);
+    }
 
     gpsd_log(&session->context->errout, LOG_DATA,
-	     "TIMEGPS: time=%.2f leap=%d, mask={TIME}\n",
-	     session->newdata.time, session->context->leap_seconds);
-    return TIME_SET | NTPTIME_IS;
+	     "TIMEGPS: time=%.2f mask={TIME}\n",
+	     session->newdata.time);
+    return mask;
 }
 
 /**
@@ -321,7 +490,7 @@ ubx_msg_nav_svinfo(struct gps_device_t *session, unsigned char *buf,
 {
     unsigned int i, nchan, nsv, st;
 
-    if (data_len < 152) {
+    if (data_len < 8) {
 	gpsd_log(&session->context->errout, LOG_PROG,
 		 "runt svinfo (datalen=%zd)\n", data_len);
 	return 0;
@@ -337,17 +506,61 @@ ubx_msg_nav_svinfo(struct gps_device_t *session, unsigned char *buf,
     nsv = 0;
     for (i = st = 0; i < nchan; i++) {
 	unsigned int off = 8 + 12 * i;
+        short PRN = (short)getub(buf, off + 1);
+        unsigned char snr = getub(buf, off + 4);
 	bool used = (bool)(getub(buf, off + 2) & 0x01);
-	if ((int)getub(buf, off + 4) == 0)
-	    continue;		/* LEA-5H seems to have a bug reporting sats it does not see or hear */
-	session->gpsdata.skyview[st].PRN = (short)getub(buf, off + 1);
-	session->gpsdata.skyview[st].ss = (float)getub(buf, off + 4);
+
+	if (0 == snr)
+	    continue;		/* skip zero SNR */
+
+        /* fit into gnssid:svid */
+	if (0 == PRN) {
+            /* skip 0 PRN */
+	    continue;
+        } else if ((1 <= PRN) && (32 >= PRN)) {
+            /* GPS */
+            session->gpsdata.skyview[st].gnssid = 0;
+            session->gpsdata.skyview[st].svid = PRN;
+        } else if ((33 <= PRN) && (64 >= PRN)) {
+            /* BeiDou */
+            session->gpsdata.skyview[st].gnssid = 3;
+            session->gpsdata.skyview[st].svid = PRN - 158;
+        } else if ((65 <= PRN) && (96 >= PRN)) {
+            /* GLONASS */
+            session->gpsdata.skyview[st].gnssid = 6;
+            session->gpsdata.skyview[st].svid = PRN - 64;
+        } else if ((120 <= PRN) && (158 >= PRN)) {
+            /* SBAS */
+            session->gpsdata.skyview[st].gnssid = 1;
+            session->gpsdata.skyview[st].svid = PRN;
+        } else if ((159 <= PRN) && (163 >= PRN)) {
+            /* BeiDou, again */
+            session->gpsdata.skyview[st].gnssid = 3;
+            session->gpsdata.skyview[st].svid = PRN - 126;
+        } else if ((173 <= PRN) && (182 >= PRN)) {
+            /* IMES */
+            session->gpsdata.skyview[st].gnssid = 4;
+            session->gpsdata.skyview[st].svid = PRN - 172;
+        } else if ((193 <= PRN) && (197 >= PRN)) {
+            /* QZSS */
+            session->gpsdata.skyview[st].gnssid = 5;
+            session->gpsdata.skyview[st].svid = PRN - 192;
+        } else if ((211 <= PRN) && (246 >= PRN)) {
+            /* Galileo */
+            session->gpsdata.skyview[st].gnssid = 2;
+            session->gpsdata.skyview[st].svid = PRN - 210;
+        } else if (255 == PRN) {
+            /* GLONASS, again */
+            session->gpsdata.skyview[st].gnssid = 6;
+            session->gpsdata.skyview[st].svid = 255;
+        }
+	session->gpsdata.skyview[st].PRN = PRN;
+
+	session->gpsdata.skyview[st].ss = (float)snr;
 	session->gpsdata.skyview[st].elevation = (short)getsb(buf, off + 5);
 	session->gpsdata.skyview[st].azimuth = (short)getles16(buf, off + 6);
 	session->gpsdata.skyview[st].used = used;
-	if (session->gpsdata.skyview[st].PRN == 0)
-	    continue;
-	if (used || session->gpsdata.skyview[st].PRN == (short)session->driver.ubx.sbas_in_use) {
+	if (used || PRN == (short)session->driver.ubx.sbas_in_use) {
 	    nsv++;
 	    session->gpsdata.skyview[st].used = true;
 	}
@@ -367,6 +580,37 @@ ubx_msg_nav_svinfo(struct gps_device_t *session, unsigned char *buf,
     return SATELLITE_SET | USED_IS;
 }
 
+/*
+ * Velocity Position ECEF message
+ */
+static gps_mask_t
+ubx_msg_nav_velecef(struct gps_device_t *session, unsigned char *buf,
+		size_t data_len)
+{
+    gps_mask_t mask = VECEF_SET;
+    double fTOW;
+
+    if (data_len < 20) {
+	gpsd_log(&session->context->errout, LOG_WARN,
+		 "Invalid NAV VELECEF message, payload len %zd", data_len);
+	return 0;
+    }
+
+    fTOW = getleu32(buf, 0) / 1000.0;
+    session->newdata.ecef.vx = getles32(buf, 4) / 100.0;
+    session->newdata.ecef.vy = getles32(buf, 8) / 100.0;
+    session->newdata.ecef.vz = getles32(buf, 12) / 100.0;
+    session->newdata.ecef.vAcc = getleu32(buf, 16) / 100.0;
+    gpsd_log(&session->context->errout, LOG_DATA,
+	"UBX_NAV_VELECEF: fTOW=%.3f ECEF vx=%.2f vy=%.2f vz=%.2f vAcc=%.2f\n",
+	fTOW,
+	session->newdata.ecef.vx,
+	session->newdata.ecef.vy,
+	session->newdata.ecef.vz,
+	session->newdata.ecef.vAcc);
+    return mask;
+}
+
 /*
  * SBAS Info
  */
@@ -462,9 +706,11 @@ gps_mask_t ubx_parse(struct gps_device_t * session, unsigned char *buf,
     /* extract message id and length */
     msgid = (buf[2] << 8) | buf[3];
     data_len = (size_t) getles16(buf, 4);
+
     switch (msgid) {
     case UBX_NAV_POSECEF:
 	gpsd_log(&session->context->errout, LOG_DATA, "UBX_NAV_POSECEF\n");
+	mask = ubx_msg_nav_posecef(session, &buf[UBX_PREFIX_LEN], data_len);
 	break;
     case UBX_NAV_POSLLH:
 	gpsd_log(&session->context->errout, LOG_DATA, "UBX_NAV_POSLLH\n");
@@ -474,15 +720,15 @@ gps_mask_t ubx_parse(struct gps_device_t * session, unsigned char *buf,
 	gpsd_log(&session->context->errout, LOG_DATA, "UBX_NAV_STATUS\n");
 	break;
     case UBX_NAV_DOP:
+        /* DOP seems to be the last NAV sent in a cycle */
 	gpsd_log(&session->context->errout, LOG_PROG, "UBX_NAV_DOP\n");
 	mask = ubx_msg_nav_dop(session, &buf[UBX_PREFIX_LEN], data_len);
 	break;
     case UBX_NAV_SOL:
         /* UBX-NAV-SOL deprecated, use UBX-NAV-PVT instead */
 	gpsd_log(&session->context->errout, LOG_PROG, "UBX_NAV_SOL\n");
-	mask =
-	    ubx_msg_nav_sol(session, &buf[UBX_PREFIX_LEN],
-			    data_len) | (CLEAR_IS | REPORT_IS);
+	mask = ubx_msg_nav_sol(session, &buf[UBX_PREFIX_LEN], data_len)
+	     | REPORT_IS;
 	break;
     case UBX_NAV_PVT:
 	gpsd_log(&session->context->errout, LOG_PROG, "UBX_NAV_PVT\n");
@@ -493,6 +739,7 @@ gps_mask_t ubx_parse(struct gps_device_t * session, unsigned char *buf,
 	break;
     case UBX_NAV_VELECEF:
 	gpsd_log(&session->context->errout, LOG_DATA, "UBX_NAV_VELECEF\n");
+	mask = ubx_msg_nav_velecef(session, &buf[UBX_PREFIX_LEN], data_len);
 	break;
     case UBX_NAV_VELNED:
 	gpsd_log(&session->context->errout, LOG_DATA, "UBX_NAV_VELNED\n");
@@ -596,9 +843,20 @@ gps_mask_t ubx_parse(struct gps_device_t * session, unsigned char *buf,
 	break;
 
     case UBX_CFG_PRT:
-	session->driver.ubx.port_id = (unsigned char)buf[UBX_MESSAGE_DATA_OFFSET + 0];
-	gpsd_log(&session->context->errout, LOG_INF, "UBX_CFG_PRT: port %d\n",
-		 session->driver.ubx.port_id);
+        if ( session->driver.ubx.port_id != (unsigned char)buf[UBX_MESSAGE_DATA_OFFSET + 0] ) {
+	    session->driver.ubx.port_id = (unsigned char)buf[UBX_MESSAGE_DATA_OFFSET + 0];
+	    gpsd_log(&session->context->errout, LOG_INF,
+		     "UBX_CFG_PRT: port %d\n", session->driver.ubx.port_id);
+
+#ifdef RECONFIGURE_ENABLE
+	    /* Need to reinitialize since port changed */
+	    if (session->mode == O_OPTIMIZE) {
+		ubx_mode(session, MODE_BINARY);
+	    } else {
+		ubx_mode(session, MODE_NMEA);
+	    }
+#endif /* RECONFIGURE_ENABLE */
+	}
 	break;
 
     case UBX_TIM_TP:
@@ -654,6 +912,10 @@ bool ubx_write(struct gps_device_t * session,
     size_t i;
     bool ok;
 
+    /* do not write if -b (readonly) option set */
+    if (session->context->readonly)
+        return true;
+
     session->msgbuf[0] = 0xb5;
     session->msgbuf[1] = 0x62;
 
@@ -705,7 +967,7 @@ static ssize_t ubx_control_send(struct gps_device_t *session, char *msg,
 
 static void ubx_init_query(struct gps_device_t *session)
 {
-    /* MON_VER: query for version information */
+    /* UBX-MON-VER: query for version information */
     (void)ubx_write(session, UBX_CLASS_MON, 0x04, NULL, 0);
 }
 
@@ -726,6 +988,7 @@ static void ubx_event_hook(struct gps_device_t *session, event_t event)
 	msg[5] = 0x00;
 	msg[6] = 0x00;
 	msg[7] = 0x00;
+        /* UBX-CFG-SBAS */
 	(void)ubx_write(session, 0x06u, 0x16, msg, 8);
 
 #ifdef RECONFIGURE_ENABLE
@@ -748,7 +1011,7 @@ static void ubx_event_hook(struct gps_device_t *session, event_t event)
 	gpsd_log(&session->context->errout, LOG_DATA, "UBX revert\n");
 
 	/* Reverting all in one fast and reliable reset */
-	(void)ubx_write(session, 0x06, 0x04, msg, 4);	/* CFG-RST */
+	(void)ubx_write(session, 0x06, 0x04, msg, 4);	/* UBX-CFG-RST */
     }
 }
 
@@ -789,9 +1052,9 @@ static void ubx_cfg_prt(struct gps_device_t *session,
      * has port ID 3 the way it ought to.
      */
     else if (strstr(session->gpsdata.dev.path, "/ACM") != NULL)
-	buf[0] = USB_ID;
+	session->driver.ubx.port_id = buf[0] = USB_ID;
     else
-	buf[0] = USART1_ID;
+	session->driver.ubx.port_id = buf[0] = USART1_ID;
 
     putle32(buf, 8, speed);
 
@@ -850,6 +1113,13 @@ static void ubx_cfg_prt(struct gps_device_t *session,
     /* enable all input protocols by default */
     buf[12] = NMEA_PROTOCOL_MASK | UBX_PROTOCOL_MASK | RTCM_PROTOCOL_MASK;
 
+    buf[outProtoMask] = (mode == MODE_NMEA
+                         ? NMEA_PROTOCOL_MASK : UBX_PROTOCOL_MASK);
+    (void)ubx_write(session, 0x06u, 0x00, buf, sizeof(buf));
+
+    gpsd_log(&session->context->errout, LOG_DATA,
+	"UBX ubx_cfg_prt mode:%d, port:%d\n", mode, buf[0]);
+
     /* selectively enable output protocols */
     if (mode == MODE_NMEA) {
 	/*
@@ -916,9 +1186,6 @@ static void ubx_cfg_prt(struct gps_device_t *session,
 	msg[1] = 0x08;		/* msg id  = ZDA */
 	msg[2] = 0x01;		/* rate */
 	(void)ubx_write(session, 0x06u, 0x01, msg, 3);
-
-	buf[outProtoMask] &= ~UBX_PROTOCOL_MASK;
-	buf[outProtoMask] |= NMEA_PROTOCOL_MASK;
     } else { /* MODE_BINARY */
 	/*
 	 * Just enabling the UBX protocol for output is not enough to
@@ -969,6 +1236,14 @@ static void ubx_cfg_prt(struct gps_device_t *session,
 	msg[1] = 0x32;		/* msg id  = NAV-SBAS */
 	msg[2] = 0x0a;		/* rate */
 	(void)ubx_write(session, 0x06u, 0x01, msg, 3);
+	msg[0] = 0x01;		/* class */
+	msg[1] = 0x01;		/* msg id  = UBX-NAV-POSECEF */
+	msg[2] = 0x01;		/* rate */
+	(void)ubx_write(session, 0x06u, 0x01, msg, 3);
+	msg[0] = 0x01;		/* class */
+	msg[1] = 0x11;		/* msg id  = UBX-NAV-VELECEF */
+	msg[2] = 0x01;		/* rate */
+	(void)ubx_write(session, 0x06u, 0x01, msg, 3);
 
 
 #ifdef __UNUSED__
@@ -1012,12 +1287,7 @@ static void ubx_cfg_prt(struct gps_device_t *session,
 	msg[2] = 0x00;		/* rate */
 	(void)ubx_write(session, 0x06u, 0x01, msg, 3);
 #endif /* __UNUSED __ */
-
-	buf[outProtoMask] &= ~NMEA_PROTOCOL_MASK;
-	buf[outProtoMask] |= UBX_PROTOCOL_MASK;
     }
-
-    (void)ubx_write(session, 0x06u, 0x00, buf, sizeof(buf));
 }
 
 static void ubx_mode(struct gps_device_t *session, int mode)
